
__author__ = "Shashank Agrawal"
__license__ = "MIT"
__version__ = "0.1.1"
__email__ = "dew@bluemist-ai.one"


from bluemist.preprocessing.preprocessor import (preprocess_data)

__all__ = [
    'preprocess_data'
]
