# Author: Shashank Agrawal
# License: MIT
# Version: 0.1.2
# Email: dew@bluemist-ai.one
# Created: Jun 22, 2022
# Last modified: May 29, 2023

from bluemist.regression import tuning
from bluemist.regression.regressor import (
    get_estimators,
    train_test_evaluate,
    deploy_model
)

__all__ = ['get_estimators',
           'train_test_evaluate',
           'deploy_model'
]
