
__author__ = "Shashank Agrawal"
__license__ = "MIT"
__version__ = "0.1.1"
__email__ = "dew@bluemist-ai.one"


__all__ = [
    'datasource',
    'eda',
    'preprocessing',
    'regression',
    'classification'
]
