
# Author: Shashank Agrawal
# License: MIT
# Version: 0.1.2
# Email: dew@bluemist-ai.one
# Created: Jun 11, 2023
# Last modified: Jun 19, 2023

GPU_BRAND_NVIDIA = 'NVIDIA'
GPU_BRAND_INTEL = 'Intel'
CPU_BRAND_INTEL = 'GenuineIntel'

GPU_ACCELERATION_NVIDIA = 'NVIDIA GPU Acceleration'
CPU_ACCELERATION_INTEL = 'Intel CPU Acceleration'
